﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.IO;
using System.Diagnostics;

namespace QuickNASM
{
    public partial class MainForm
    {
        // constants

        // fields
        string filename = "";

        // methods
        void Log(string text)
        {
            LogBox.Text += (text + Environment.NewLine);
            LogBox.SelectionStart = LogBox.Text.Length;
            LogBox.ScrollToCaret();
        }

        #region Private Event Handler List
        // 이벤트 핸들러의 목록을 정의합니다.
        void CreateFile()
        {
            
        }
        void OpenFile()
        {
            OpenFileDialog dialog = new OpenFileDialog();
            if (dialog.ShowDialog() != DialogResult.OK)
                return;
            StreamReader reader = new StreamReader(dialog.FileName);
            CodeBox.Text = reader.ReadToEnd();
            reader.Close();
        }

        bool Save()
        {
            if (filename == string.Empty)
            {
                SaveFileDialog dialog = new SaveFileDialog();
                dialog.InitialDirectory = Environment.CurrentDirectory;
                dialog.DefaultExt = ".asm";
                if (dialog.ShowDialog() != DialogResult.OK)
                    return false;
                filename = dialog.FileName;
            }
            StreamWriter writer = new StreamWriter(filename);
            writer.Write(CodeBox.Text);
            writer.Close();

            Log("Save complete");
            return true;
        }
        void SaveAs()
        {

        }
        void CloseForm()
        {
            this.Close();
        }

        void Undo()
        {
            CodeBox.Undo();
        }
        void Redo()
        {
            CodeBox.Redo();
        }

        void CutText()
        {
            CodeBox.Cut();
        }
        void CopyText()
        {
            CodeBox.Copy();
        }
        void PasteText()
        {
            CodeBox.Paste();
        }
        void DeleteText()
        {
            if (CodeBox.SelectionLength > 0)
            {
                CodeBox.Text = CodeBox.Text.Remove(CodeBox.SelectionStart, CodeBox.SelectionLength);
            }
            else if (CodeBox.SelectionStart < CodeBox.Text.Length)
            {
                CodeBox.Text = CodeBox.Text.Remove(CodeBox.SelectionStart, 1);
            }
        }
        void SelectAllText()
        {
            CodeBox.SelectAll();
        }

        bool Assemble()
        {
            try
            {
                if (Save() == false)
                    throw new Exception("cannot save file " + filename);

                string assemblyName = Handy.String.GetFileName(filename);
                string nasmPath = "C:\\Users\\HandyMain\\AppData\\Local\\nasm\\nasm";
                string commandParam = "-f win32 " + filename;
                Process proc = Process.Start(nasmPath, commandParam);
                proc.WaitForExit();

                if (proc.ExitCode != 0)
                    throw new Exception("process did not end with code 0");

                Log("Assemble complete");
                return true;
            }
            catch (Exception ex)
            {
                Log(ex.Message);
                return false;
            }
        }
        bool Link()
        {
            try
            {
                string linkerPath = "C:\\MinGW\\bin\\gcc.exe";
                string execName = "out";
                string outputPath = Environment.CurrentDirectory + "\\" + execName + ".exe";
                string objectPath = Environment.CurrentDirectory + "\\" + Handy.String.GetFileName(filename) + ".obj";
                string commandParam = string.Format("{1}", outputPath, objectPath);
                Process proc = Process.Start(linkerPath, commandParam);
                proc.WaitForExit();

                if (proc.ExitCode != 0)
                    throw new Exception("process did not end with code 0");

                Log("Link complete");
                return true;
            }
            catch (Exception ex)
            {
                Log(ex.Message);
                return false;
            }
        }
        bool Build()
        {
            try
            {
                if (Assemble() == false)
                    throw new Exception("Assemble failed");
                if (Link() == false)
                    throw new Exception("Linked failed");
                Log("Build complete");
                return true;
            }
            catch (Exception ex)
            {
                Log(ex.Message);
                return false;
            }
        }
        void Run()
        {
            Log("Running");
        }
        void BuildAndRun()
        {
            Build();
            Run();
        }

        #endregion Private Event Handler List
    }
}
